"""Run with this config by doing python -m example.main --config super_agi.py"""
from sws import Config, lazy


def get_config():
    c = Config()
    c.lr = 0.0003
    c.wd = lazy(lambda c: c.lr * 0.1)
    c.model.depth = 64
    c.model.heads = 96
    c.model.width = lazy(lambda c: c.heads * 128)
    c.model.init_seed = 42
    return c



"""Minimal example using the sws package.

To run this, either install sws, or do python -m examples.main
"""

import os
import sys

from sws import Config, lazy, run


def get_config():
    c = Config()
    c.lr = 0.001
    c.wd = lazy(lambda c: c.lr * 0.1)
    c.model.depth = 4
    c.model.width = 256
    c.model.heads = lazy(lambda m: 4 if m.width > 128 else 1)
    return c


def make_model(depth, width, heads, **secret):
    print(f"ModelToAGI(d={depth}, wh={width // heads}, nh={heads})")
    if secret:
        print(f"/!\\ The model is using secret sauce {secret}")


def main(c):
    print("Running AGI training with the full config:")
    print(c)
    make_model(**c.model.to_dict())


if __name__ == "__main__":
    run(main)

    # Alternate, a bit more explicit and a bit less flexible:
    # main(get_config().finalize(sys.argv[1:]))
    # NOTE: This doesn't allow to change config file with --config.



import sys
from sws import run


def get_config():
    from sws import Config
    c = Config(lr=0.1)
    return c


def main(cfg, rest):
    # Print values in a simple, parseable way
    print(f"lr={cfg.lr}")
    print("unused=" + ",".join(rest))


if __name__ == "__main__":
    run(main, forward_extras=True)
